@@ -19,18 +19,19 @@
 
 package org.apache.comet.shims
 
+import org.apache.spark.paths.SparkPath
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.execution.datasources.PartitionedFile
 
 object ShimBatchReader {
-
   def newPartitionedFile(partitionValues: InternalRow, file: String): PartitionedFile =
     PartitionedFile(
       partitionValues,
-      file,
+      SparkPath.fromUrlString(file),
       -1, // -1 means we read the entire file
       -1,
       Array.empty[String],
       0,
-      0)
+      0
+    )
 }
