@@ -23,4 +23,14 @@ rm -rf build 2> /dev/null
 rm -rf temp 2> /dev/null
 mkdir temp
 cp -rf source/* temp/
+
+# Add user guide from published releases
+rm -rf comet-0.8
+rm -rf comet-0.9
+python3 generate-versions.py
+
+# Remove overview pages (this used to be part of the user guide but is now a top level page)
+rm temp/user-guide/0.9/overview.md
+rm temp/user-guide/0.8/overview.md
+
 make SOURCEDIR=`pwd`/temp html
