@@ -16,12 +16,13 @@
 // under the License.
 
 pub(crate) mod codec;
+mod comet_partitioning;
 mod list;
 mod map;
+mod range_partitioner;
 pub mod row;
 mod shuffle_writer;
 
-pub use codec::BatchWriter;
-
 pub use codec::{read_ipc_compressed, CompressionCodec, ShuffleBlockWriter};
+pub use comet_partitioning::CometPartitioning;
 pub use shuffle_writer::ShuffleWriterExec;
