@@ -18,6 +18,7 @@
 #![allow(incomplete_features)]
 #![allow(non_camel_case_types)]
 #![allow(clippy::upper_case_acronyms)]
+#![allow(clippy::result_large_err)]
 // For prost generated struct
 #![allow(clippy::derive_partial_eq_without_eq)]
 // The clippy throws an error if the reference clone not wrapped into `Arc::clone`
@@ -36,9 +37,13 @@ use log4rs::{
     encode::pattern::PatternEncoder,
     Config,
 };
+use once_cell::sync::OnceCell;
+
+#[cfg(all(not(target_env = "msvc"), feature = "jemalloc"))]
+use tikv_jemallocator::Jemalloc;
+
 #[cfg(feature = "mimalloc")]
 use mimalloc::MiMalloc;
-use once_cell::sync::OnceCell;
 
 use errors::{try_unwrap_or_throw, CometError, CometResult};
 
@@ -50,6 +55,10 @@ pub mod execution;
 mod jvm_bridge;
 pub mod parquet;
 
+#[cfg(all(not(target_env = "msvc"), feature = "jemalloc"))]
+#[global_allocator]
+static GLOBAL: Jemalloc = Jemalloc;
+
 #[cfg(feature = "mimalloc")]
 #[global_allocator]
 static GLOBAL: MiMalloc = MiMalloc;
@@ -84,7 +93,7 @@ pub extern "system" fn Java_org_apache_comet_NativeBase_init(
         JAVA_VM.get_or_init(|| java_vm);
 
         let comet_version = env!("CARGO_PKG_VERSION");
-        info!("Comet native library version {} initialized", comet_version);
+        info!("Comet native library version {comet_version} initialized");
         Ok(())
     })
 }
