@@ -15,15 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::utils::{is_valid_decimal_precision, unlikely};
+use crate::utils::{build_bool_state, is_valid_decimal_precision};
 use arrow::array::{
     cast::AsArray, types::Decimal128Type, Array, ArrayRef, BooleanArray, Decimal128Array,
 };
 use arrow::datatypes::{DataType, Field, FieldRef};
-use arrow::{
-    array::BooleanBufferBuilder,
-    buffer::{BooleanBuffer, NullBuffer},
-};
+use arrow::{array::BooleanBufferBuilder, buffer::NullBuffer};
 use datafusion::common::{DataFusionError, Result as DFResult, ScalarValue};
 use datafusion::logical_expr::function::{AccumulatorArgs, StateFieldsArgs};
 use datafusion::logical_expr::Volatility::Immutable;
@@ -290,24 +287,16 @@ impl SumDecimalGroupsAccumulator {
         !self.is_empty.get_bit(index) && !self.is_not_null.get_bit(index)
     }
 
+    #[inline]
     fn update_single(&mut self, group_index: usize, value: i128) {
-        if unlikely(self.is_overflow(group_index)) {
-            // This means there's a overflow in decimal, so we will just skip the rest
-            // of the computation
-            return;
-        }
-
         self.is_empty.set_bit(group_index, false);
         let (new_sum, is_overflow) = self.sum[group_index].overflowing_add(value);
+        self.sum[group_index] = new_sum;
 
         if is_overflow || !is_valid_decimal_precision(new_sum, self.precision) {
             // Overflow: set buffer accumulator to null
             self.is_not_null.set_bit(group_index, false);
-            return;
         }
-
-        self.sum[group_index] = new_sum;
-        self.is_not_null.set_bit(group_index, true)
     }
 }
 
@@ -318,25 +307,6 @@ fn ensure_bit_capacity(builder: &mut BooleanBufferBuilder, capacity: usize) {
     }
 }
 
-/// Build a boolean buffer from the state and reset the state, based on the emit_to
-/// strategy.
-fn build_bool_state(state: &mut BooleanBufferBuilder, emit_to: &EmitTo) -> BooleanBuffer {
-    let bool_state: BooleanBuffer = state.finish();
-
-    match emit_to {
-        EmitTo::All => bool_state,
-        EmitTo::First(n) => {
-            // split off the first N values in bool_state
-            let first_n_bools: BooleanBuffer = bool_state.iter().take(*n).collect();
-            // reset the existing seen buffer
-            for seen in bool_state.iter().skip(*n) {
-                state.append(seen);
-            }
-            first_n_bools
-        }
-    }
-}
-
 impl GroupsAccumulator for SumDecimalGroupsAccumulator {
     fn update_batch(
         &mut self,
