@@ -15,6 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod bitwise_count;
+mod bitwise_get;
 mod bitwise_not;
 
-pub use bitwise_not::{bitwise_not, BitwiseNotExpr};
+pub use bitwise_count::SparkBitwiseCount;
+pub use bitwise_get::SparkBitwiseGet;
+pub use bitwise_not::SparkBitwiseNot;
