@@ -20,7 +20,16 @@
 #[macro_export]
 macro_rules! hash_array {
     ($array_type: ident, $column: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
         if array.null_count() == 0 {
             for (i, hash) in $hashes.iter_mut().enumerate() {
                 *hash = $hash_method(&array.value(i), *hash);
@@ -38,7 +47,16 @@ macro_rules! hash_array {
 #[macro_export]
 macro_rules! hash_array_boolean {
     ($array_type: ident, $column: ident, $hash_input_type: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
         if array.null_count() == 0 {
             for (i, hash) in $hashes.iter_mut().enumerate() {
                 *hash = $hash_method($hash_input_type::from(array.value(i)).to_le_bytes(), *hash);
@@ -57,7 +75,16 @@ macro_rules! hash_array_boolean {
 #[macro_export]
 macro_rules! hash_array_primitive {
     ($array_type: ident, $column: ident, $ty: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
         let values = array.values();
 
         if array.null_count() == 0 {
@@ -77,7 +104,16 @@ macro_rules! hash_array_primitive {
 #[macro_export]
 macro_rules! hash_array_primitive_float {
     ($array_type: ident, $column: ident, $ty: ident, $ty2: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
         let values = array.values();
 
         if array.null_count() == 0 {
@@ -107,17 +143,35 @@ macro_rules! hash_array_primitive_float {
 #[macro_export]
 macro_rules! hash_array_small_decimal {
     ($array_type:ident, $column: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
 
         if array.null_count() == 0 {
             for (i, hash) in $hashes.iter_mut().enumerate() {
-                *hash = $hash_method(i64::try_from(array.value(i)).unwrap().to_le_bytes(), *hash);
+                *hash = $hash_method(
+                    i64::try_from(array.value(i))
+                        .map(|v| v.to_le_bytes())
+                        .map_err(|e| DataFusionError::Execution(e.to_string()))?,
+                    *hash,
+                );
             }
         } else {
             for (i, hash) in $hashes.iter_mut().enumerate() {
                 if !array.is_null(i) {
-                    *hash =
-                        $hash_method(i64::try_from(array.value(i)).unwrap().to_le_bytes(), *hash);
+                    *hash = $hash_method(
+                        i64::try_from(array.value(i))
+                            .map(|v| v.to_le_bytes())
+                            .map_err(|e| DataFusionError::Execution(e.to_string()))?,
+                        *hash,
+                    );
                 }
             }
         }
@@ -127,7 +181,16 @@ macro_rules! hash_array_small_decimal {
 #[macro_export]
 macro_rules! hash_array_decimal {
     ($array_type:ident, $column: ident, $hashes: ident, $hash_method: ident) => {
-        let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
+        let array = $column
+            .as_any()
+            .downcast_ref::<$array_type>()
+            .unwrap_or_else(|| {
+                panic!(
+                    "Failed to downcast column to {}. Actual data type: {:?}.",
+                    stringify!($array_type),
+                    $column.data_type()
+                )
+            });
 
         if array.null_count() == 0 {
             for (i, hash) in $hashes.iter_mut().enumerate() {
@@ -155,7 +218,7 @@ macro_rules! hash_array_decimal {
 macro_rules! create_hashes_internal {
     ($arrays: ident, $hashes_buffer: ident, $hash_method: ident, $create_dictionary_hash_method: ident) => {
         use arrow::datatypes::{DataType, TimeUnit};
-        use arrow_array::{types::*, *};
+        use arrow::array::{types::*, *};
 
         for (i, col) in $arrays.iter().enumerate() {
             let first_col = i == 0;
