@@ -49,8 +49,13 @@ mod datetime_funcs;
 pub use agg_funcs::*;
 
 pub use cast::{spark_cast, Cast, SparkCastOptions};
+
+mod bloom_filter;
+pub use bloom_filter::{BloomFilterAgg, BloomFilterMightContain};
+
 mod conditional_funcs;
 mod conversion_funcs;
+mod map_funcs;
 mod math_funcs;
 mod nondetermenistic_funcs;
 
@@ -69,9 +74,9 @@ pub use error::{SparkError, SparkResult};
 pub use hash_funcs::*;
 pub use json_funcs::ToJson;
 pub use math_funcs::{
-    create_negate_expr, spark_ceil, spark_decimal_div, spark_decimal_integral_div, spark_floor,
-    spark_hex, spark_make_decimal, spark_round, spark_unhex, spark_unscaled_value, CheckOverflow,
-    NegativeExpr, NormalizeNaNAndZero,
+    create_modulo_expr, create_negate_expr, spark_ceil, spark_decimal_div,
+    spark_decimal_integral_div, spark_floor, spark_hex, spark_make_decimal, spark_round,
+    spark_unhex, spark_unscaled_value, CheckOverflow, NegativeExpr, NormalizeNaNAndZero,
 };
 pub use string_funcs::*;
 
@@ -99,3 +104,7 @@ pub(crate) fn arithmetic_overflow_error(from_type: &str) -> SparkError {
         from_type: from_type.to_string(),
     }
 }
+
+pub(crate) fn divide_by_zero_error() -> SparkError {
+    SparkError::DivideByZero
+}
