@@ -16,10 +16,12 @@
 // under the License.
 
 mod ceil;
+pub(crate) mod checked_arithmetic;
 mod div;
 mod floor;
 pub(crate) mod hex;
 pub mod internal;
+pub mod modulo_expr;
 mod negative;
 mod round;
 pub(crate) mod unhex;
@@ -31,6 +33,7 @@ pub use div::spark_decimal_integral_div;
 pub use floor::spark_floor;
 pub use hex::spark_hex;
 pub use internal::*;
+pub use modulo_expr::create_modulo_expr;
 pub use negative::{create_negate_expr, NegativeExpr};
 pub use round::spark_round;
 pub use unhex::spark_unhex;
