@@ -17,4 +17,4 @@
 
 mod read_side_padding;
 
-pub use read_side_padding::spark_read_side_padding;
+pub use read_side_padding::{spark_read_side_padding, spark_rpad};
