@@ -19,12 +19,14 @@
 
 package org.apache.comet.parquet
 
+import java.io.ByteArrayOutputStream
 import java.lang.{Boolean => JBoolean, Byte => JByte, Double => JDouble, Float => JFloat, Long => JLong, Short => JShort}
 import java.math.{BigDecimal => JBigDecimal}
 import java.sql.{Date, Timestamp}
 import java.time.{Duration, Instant, LocalDate, Period}
 import java.util.Locale
 
+import scala.collection.JavaConverters._
 import scala.collection.JavaConverters.asScalaBufferConverter
 
 import org.apache.parquet.column.statistics.{Statistics => ParquetStatistics}
@@ -39,9 +41,12 @@ import org.apache.parquet.schema.Type.Repetition
 import org.apache.spark.sql.catalyst.util.{quoteIfNeeded, CaseInsensitiveMap, DateTimeUtils, IntervalUtils}
 import org.apache.spark.sql.catalyst.util.RebaseDateTime.{rebaseGregorianToJulianDays, rebaseGregorianToJulianMicros, RebaseSpec}
 import org.apache.spark.sql.sources
+import org.apache.spark.sql.types.StructType
 import org.apache.spark.unsafe.types.UTF8String
 
-import org.apache.comet.CometSparkSessionExtensions.isSpark34Plus
+import org.apache.comet.parquet.SourceFilterSerde.{createBinaryExpr, createNameExpr, createUnaryExpr, createValueExpr}
+import org.apache.comet.serde.ExprOuterClass
+import org.apache.comet.serde.QueryPlanSerde.scalarFunctionExprToProto
 import org.apache.comet.shims.ShimSQLConf
 
 /**
@@ -52,6 +57,7 @@ import org.apache.comet.shims.ShimSQLConf
  */
 class ParquetFilters(
     schema: MessageType,
+    dataSchema: StructType,
     pushDownDate: Boolean,
     pushDownTimestamp: Boolean,
     pushDownDecimal: Boolean,
@@ -620,17 +626,12 @@ class ParquetFilters(
     value == null || (nameToParquetField(name).fieldType match {
       case ParquetBooleanType => value.isInstanceOf[JBoolean]
       case ParquetByteType | ParquetShortType | ParquetIntegerType =>
-        if (isSpark34Plus) {
-          value match {
-            // Byte/Short/Int are all stored as INT32 in Parquet so filters are built using type
-            // Int. We don't create a filter if the value would overflow.
-            case _: JByte | _: JShort | _: Integer => true
-            case v: JLong => v.longValue() >= Int.MinValue && v.longValue() <= Int.MaxValue
-            case _ => false
-          }
-        } else {
-          // If not Spark 3.4+, we still following the old behavior as Spark does.
-          value.isInstanceOf[Number]
+        value match {
+          // Byte/Short/Int are all stored as INT32 in Parquet so filters are built using type
+          // Int. We don't create a filter if the value would overflow.
+          case _: JByte | _: JShort | _: Integer => true
+          case v: JLong => v.longValue() >= Int.MinValue && v.longValue() <= Int.MaxValue
+          case _ => false
         }
       case ParquetLongType => value.isInstanceOf[JLong]
       case ParquetFloatType => value.isInstanceOf[JFloat]
@@ -723,20 +724,22 @@ class ParquetFilters(
           .lift(nameToParquetField(name).fieldType)
           .map(_(nameToParquetField(name).fieldNames, value))
 
-      case sources.LessThan(name, value) if canMakeFilterOn(name, value) =>
+      case sources.LessThan(name, value) if (value != null) && canMakeFilterOn(name, value) =>
         makeLt
           .lift(nameToParquetField(name).fieldType)
           .map(_(nameToParquetField(name).fieldNames, value))
-      case sources.LessThanOrEqual(name, value) if canMakeFilterOn(name, value) =>
+      case sources.LessThanOrEqual(name, value)
+          if (value != null) && canMakeFilterOn(name, value) =>
         makeLtEq
           .lift(nameToParquetField(name).fieldType)
           .map(_(nameToParquetField(name).fieldNames, value))
 
-      case sources.GreaterThan(name, value) if canMakeFilterOn(name, value) =>
+      case sources.GreaterThan(name, value) if (value != null) && canMakeFilterOn(name, value) =>
         makeGt
           .lift(nameToParquetField(name).fieldType)
           .map(_(nameToParquetField(name).fieldNames, value))
-      case sources.GreaterThanOrEqual(name, value) if canMakeFilterOn(name, value) =>
+      case sources.GreaterThanOrEqual(name, value)
+          if (value != null) && canMakeFilterOn(name, value) =>
         makeGtEq
           .lift(nameToParquetField(name).fieldType)
           .map(_(nameToParquetField(name).fieldNames, value))
@@ -880,4 +883,160 @@ class ParquetFilters(
       case _ => None
     }
   }
+
+  def createNativeFilters(predicates: Seq[sources.Filter]): Option[Array[Byte]] = {
+    predicates.reduceOption(sources.And).flatMap(createNativeFilter).map { expr =>
+      val outputStream = new ByteArrayOutputStream()
+      expr.writeTo(outputStream)
+      outputStream.close()
+      outputStream.toByteArray
+    }
+  }
+
+  private def createNativeFilter(predicate: sources.Filter): Option[ExprOuterClass.Expr] = {
+    def nameUnaryExpr(name: String)(
+        f: (ExprOuterClass.Expr.Builder, ExprOuterClass.UnaryExpr) => ExprOuterClass.Expr.Builder)
+        : Option[ExprOuterClass.Expr] = {
+      createNameExpr(name, dataSchema).map { case (_, childExpr) =>
+        createUnaryExpr(childExpr, f)
+      }
+    }
+
+    def nameValueBinaryExpr(name: String, value: Any)(
+        f: (
+            ExprOuterClass.Expr.Builder,
+            ExprOuterClass.BinaryExpr) => ExprOuterClass.Expr.Builder)
+        : Option[ExprOuterClass.Expr] = {
+      createNameExpr(name, dataSchema).flatMap { case (dataType, childExpr) =>
+        createValueExpr(value, dataType).map(createBinaryExpr(childExpr, _, f))
+      }
+    }
+
+    predicate match {
+      case sources.IsNull(name) if canMakeFilterOn(name, null) =>
+        nameUnaryExpr(name) { (builder, unaryExpr) =>
+          builder.setIsNull(unaryExpr)
+        }
+      case sources.IsNotNull(name) if canMakeFilterOn(name, null) =>
+        nameUnaryExpr(name) { (builder, unaryExpr) =>
+          builder.setIsNotNull(unaryExpr)
+        }
+
+      case sources.EqualTo(name, value) if canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setEq(binaryExpr)
+        }
+
+      case sources.Not(sources.EqualTo(name, value)) if canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setNeq(binaryExpr)
+        }
+
+      case sources.EqualNullSafe(name, value) if canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setEqNullSafe(binaryExpr)
+        }
+
+      case sources.Not(sources.EqualNullSafe(name, value)) if canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setNeqNullSafe(binaryExpr)
+        }
+
+      case sources.LessThan(name, value) if (value != null) && canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setLt(binaryExpr)
+        }
+
+      case sources.LessThanOrEqual(name, value)
+          if (value != null) && canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setLtEq(binaryExpr)
+        }
+
+      case sources.GreaterThan(name, value) if (value != null) && canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setGt(binaryExpr)
+        }
+
+      case sources.GreaterThanOrEqual(name, value)
+          if (value != null) && canMakeFilterOn(name, value) =>
+        nameValueBinaryExpr(name, value) { (builder, binaryExpr) =>
+          builder.setGtEq(binaryExpr)
+        }
+
+      case sources.And(lhs, rhs) =>
+        (createNativeFilter(lhs), createNativeFilter(rhs)) match {
+          case (Some(leftExpr), Some(rightExpr)) =>
+            Some(
+              createBinaryExpr(
+                leftExpr,
+                rightExpr,
+                (builder, binaryExpr) => builder.setAnd(binaryExpr)))
+          case _ => None
+        }
+
+      case sources.Or(lhs, rhs) =>
+        (createNativeFilter(lhs), createNativeFilter(rhs)) match {
+          case (Some(leftExpr), Some(rightExpr)) =>
+            Some(
+              createBinaryExpr(
+                leftExpr,
+                rightExpr,
+                (builder, binaryExpr) => builder.setOr(binaryExpr)))
+          case _ => None
+        }
+
+      case sources.Not(pred) =>
+        val childExpr = createNativeFilter(pred)
+        childExpr.map { expr =>
+          createUnaryExpr(expr, (builder, unaryExpr) => builder.setNot(unaryExpr))
+        }
+
+      case sources.In(name, values)
+          if pushDownInFilterThreshold > 0 && values.nonEmpty &&
+            canMakeFilterOn(name, values.head) =>
+        createNameExpr(name, dataSchema).flatMap { case (dataType, nameExpr) =>
+          val valueExprs = values.flatMap(createValueExpr(_, dataType))
+          if (valueExprs.length != values.length) {
+            None
+          } else {
+            val builder = ExprOuterClass.In.newBuilder()
+            builder.setInValue(nameExpr)
+            builder.addAllLists(valueExprs.toSeq.asJava)
+            builder.setNegated(false)
+            Some(
+              ExprOuterClass.Expr
+                .newBuilder()
+                .setIn(builder)
+                .build())
+          }
+        }
+
+      case sources.StringStartsWith(attribute, prefix)
+          if pushDownStringPredicate && canMakeFilterOn(attribute, prefix) =>
+        val attributeExpr = createNameExpr(attribute, dataSchema)
+        val prefixExpr = attributeExpr.flatMap { case (dataType, _) =>
+          createValueExpr(prefix, dataType)
+        }
+        scalarFunctionExprToProto("starts_with", Some(attributeExpr.get._2), prefixExpr)
+
+      case sources.StringEndsWith(attribute, suffix)
+          if pushDownStringPredicate && canMakeFilterOn(attribute, suffix) =>
+        val attributeExpr = createNameExpr(attribute, dataSchema)
+        val suffixExpr = attributeExpr.flatMap { case (dataType, _) =>
+          createValueExpr(suffix, dataType)
+        }
+        scalarFunctionExprToProto("ends_with", Some(attributeExpr.get._2), suffixExpr)
+
+      case sources.StringContains(attribute, value)
+          if pushDownStringPredicate && canMakeFilterOn(attribute, value) =>
+        val attributeExpr = createNameExpr(attribute, dataSchema)
+        val valueExpr = attributeExpr.flatMap { case (dataType, _) =>
+          createValueExpr(value, dataType)
+        }
+        scalarFunctionExprToProto("contains", Some(attributeExpr.get._2), valueExpr)
+
+      case _ => None
+    }
+  }
 }
