@@ -36,8 +36,6 @@ import com.google.common.base.Objects
  *
  * Similar to `CometTakeOrderedAndProjectExec`, it contains two native executions seperated by a
  * Comet shuffle.
- *
- * TODO: support offset semantics
  */
 case class CometCollectLimitExec(
     override val originalPlan: SparkPlan,
@@ -57,13 +55,17 @@ case class CometCollectLimitExec(
     "dataSize" -> SQLMetrics.createSizeMetric(sparkContext, "data size"),
     "numPartitions" -> SQLMetrics.createMetric(
       sparkContext,
-      "number of partitions")) ++ readMetrics ++ writeMetrics
+      "number of partitions")) ++ readMetrics ++ writeMetrics ++ CometMetricNode.shuffleMetrics(
+    sparkContext)
 
   private lazy val serializer: Serializer =
     new UnsafeRowSerializer(child.output.size, longMetric("dataSize"))
 
   override def executeCollect(): Array[InternalRow] = {
-    ColumnarToRowExec(child).executeTake(limit)
+    val rows =
+      if (limit >= 0) ColumnarToRowExec(child).executeTake(limit)
+      else ColumnarToRowExec(child).executeCollect()
+    if (offset > 0) rows.drop(offset) else rows
   }
 
   protected override def doExecuteColumnar(): RDD[ColumnarBatch] = {
@@ -90,7 +92,7 @@ case class CometCollectLimitExec(
 
         new CometShuffledBatchRDD(dep, readMetrics)
       }
-      CometExecUtils.getNativeLimitRDD(singlePartitionRDD, output, limit)
+      CometExecUtils.getNativeLimitRDD(singlePartitionRDD, output, limit, offset)
     }
   }
 
