@@ -24,11 +24,14 @@ import org.apache.spark.sql._
 import org.apache.spark.sql.internal.SQLConf
 
 class CometSparkSessionExtensionsSuite extends CometTestBase {
-  test("isCometEnabled") {
+
+  import CometSparkSessionExtensions._
+
+  test("isCometLoaded") {
     val conf = new SQLConf
 
     conf.setConfString(CometConf.COMET_ENABLED.key, "false")
-    assert(!CometSparkSessionExtensions.isCometEnabled(conf))
+    assert(!isCometLoaded(conf))
 
     // Since the native lib is probably already loaded due to previous tests, we reset it here
     NativeBase.setLoaded(false)
@@ -36,12 +39,12 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
     conf.setConfString(CometConf.COMET_ENABLED.key, "true")
     val oldProperty = System.getProperty("os.name")
     System.setProperty("os.name", "foo")
-    assert(!CometSparkSessionExtensions.isCometEnabled(conf))
+    assert(!isCometLoaded(conf))
 
     System.setProperty("os.name", oldProperty)
 
     conf.setConf(SQLConf.PARQUET_INT96_TIMESTAMP_CONVERSION, true)
-    assert(!CometSparkSessionExtensions.isCometEnabled(conf))
+    assert(!isCometLoaded(conf))
 
     // Restore the original state
     NativeBase.setLoaded(true)
@@ -75,9 +78,7 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
 
   test("Minimum Comet memory overhead") {
     val conf = new SparkConf()
-    assert(
-      CometSparkSessionExtensions
-        .getCometMemoryOverhead(conf) == getBytesFromMib(384))
+    assert(getCometMemoryOverhead(conf) == getBytesFromMib(384))
   }
 
   test("Comet memory overhead factor with executor memory") {
@@ -85,25 +86,41 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
     sparkConf.set("spark.executor.memory", "16g")
     sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD_FACTOR.key, "0.5")
 
-    assert(
-      CometSparkSessionExtensions
-        .getCometMemoryOverhead(sparkConf) == getBytesFromMib(8 * 1024))
+    assert(getCometMemoryOverhead(sparkConf) == getBytesFromMib(8 * 1024))
   }
 
   test("Comet memory overhead factor with default executor memory") {
     val sparkConf = new SparkConf()
     sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD_FACTOR.key, "0.5")
-    assert(
-      CometSparkSessionExtensions
-        .getCometMemoryOverhead(sparkConf) == getBytesFromMib(512))
+    assert(getCometMemoryOverhead(sparkConf) == getBytesFromMib(512))
   }
 
   test("Comet memory overhead") {
     val sparkConf = new SparkConf()
     sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "10g")
-    assert(
-      CometSparkSessionExtensions
-        .getCometMemoryOverhead(sparkConf) == getBytesFromMib(1024 * 10))
+    assert(getCometMemoryOverhead(sparkConf) == getBytesFromMib(1024 * 10))
+    assert(shouldOverrideMemoryConf(sparkConf))
+  }
+
+  test("Comet memory overhead (min)") {
+    val sparkConf = new SparkConf()
+    sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD_MIN_MIB.key, "2g")
+    assert(getCometMemoryOverhead(sparkConf) == getBytesFromMib(1024 * 2))
+  }
+
+  test("Comet memory overhead (factor)") {
+    val sparkConf = new SparkConf()
+    sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD_FACTOR.key, "0.5")
+    assert(getCometMemoryOverhead(sparkConf) == getBytesFromMib(512))
+  }
+
+  test("Comet memory overhead (off heap)") {
+    val sparkConf = new SparkConf()
+    sparkConf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "64g")
+    sparkConf.set("spark.memory.offHeap.enabled", "true")
+    sparkConf.set("spark.memory.offHeap.size", "10g")
+    assert(getCometMemoryOverhead(sparkConf) == 0)
+    assert(!shouldOverrideMemoryConf(sparkConf))
   }
 
   test("Comet shuffle memory factor") {
@@ -114,12 +131,12 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
 
     // Minimum Comet memory overhead is 384MB
     assert(
-      CometSparkSessionExtensions.getCometShuffleMemorySize(conf, sqlConf) ==
+      getCometShuffleMemorySize(conf, sqlConf) ==
         getBytesFromMib((384 * 0.2).toLong))
 
     conf.set(CometConf.COMET_MEMORY_OVERHEAD_FACTOR.key, "0.5")
     assert(
-      CometSparkSessionExtensions.getCometShuffleMemorySize(conf, sqlConf) ==
+      getCometShuffleMemorySize(conf, sqlConf) ==
         getBytesFromMib((1024 * 0.5 * 0.2).toLong))
   }
 
@@ -129,9 +146,20 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "1g")
     sqlConf.setConfString(CometConf.COMET_COLUMNAR_SHUFFLE_MEMORY_SIZE.key, "512m")
 
-    assert(
-      CometSparkSessionExtensions
-        .getCometShuffleMemorySize(conf, sqlConf) == getBytesFromMib(512))
+    assert(getCometShuffleMemorySize(conf, sqlConf) == getBytesFromMib(512))
+  }
+
+  test("Comet shuffle memory (off-heap)") {
+    val conf = new SparkConf()
+    val sqlConf = new SQLConf
+    conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "1g")
+    conf.set("spark.memory.offHeap.enabled", "true")
+    conf.set("spark.memory.offHeap.size", "10g")
+    sqlConf.setConfString(CometConf.COMET_COLUMNAR_SHUFFLE_MEMORY_SIZE.key, "512m")
+
+    assertThrows[AssertionError] {
+      getCometShuffleMemorySize(conf, sqlConf)
+    }
   }
 
   test("Comet shuffle memory cannot be larger than Comet memory overhead") {
@@ -139,8 +167,6 @@ class CometSparkSessionExtensionsSuite extends CometTestBase {
     val sqlConf = new SQLConf
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "1g")
     sqlConf.setConfString(CometConf.COMET_COLUMNAR_SHUFFLE_MEMORY_SIZE.key, "10g")
-    assert(
-      CometSparkSessionExtensions
-        .getCometShuffleMemorySize(conf, sqlConf) == getBytesFromMib(1024))
+    assert(getCometShuffleMemorySize(conf, sqlConf) == getBytesFromMib(1024))
   }
 }
