@@ -27,7 +27,7 @@ import org.apache.spark.sql.catalyst.optimizer.EliminateSorts
 import org.apache.spark.sql.comet.CometHashAggregateExec
 import org.apache.spark.sql.comet.execution.shuffle.CometShuffleExchangeExec
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
-import org.apache.spark.sql.functions.{count_distinct, sum}
+import org.apache.spark.sql.functions.{avg, count_distinct, sum}
 import org.apache.spark.sql.internal.SQLConf
 
 import org.apache.comet.CometConf
@@ -279,6 +279,25 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
+  test("AVG decimal supports emit.first") {
+    withSQLConf(
+      SQLConf.OPTIMIZER_EXCLUDED_RULES.key -> EliminateSorts.ruleName,
+      CometConf.COMET_ENABLED.key -> "true",
+      CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
+      CometConf.COMET_SHUFFLE_MODE.key -> "jvm") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test")
+          makeParquetFile(path, 10000, 10, dictionaryEnabled)
+          withParquetTable(path.toUri.toString, "tbl") {
+            checkSparkAnswerAndOperator(
+              sql("SELECT * FROM tbl").sort("_g1").groupBy("_g1").agg(avg("_8")))
+          }
+        }
+      }
+    }
+  }
+
   test("Fix NPE in partial decimal sum") {
     val table = "tbl"
     withTable(table) {
@@ -958,8 +977,8 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
-  test("sum overflow on decimal(38, _)") {
-    withSQLConf(CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true") {
+  test("avg/sum overflow on decimal(38, _)") {
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
       val table = "overflow_decimal_38"
       withTable(table) {
         sql(s"create table $table(a decimal(38, 2), b INT) using parquet")
@@ -968,7 +987,9 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         sql(s"insert into $table values(42.00, 2), (99999999999999999999999999999999.99, 2)")
         sql(s"insert into $table values(999999999999999999999999999999999999.99, 3)")
         sql(s"insert into $table values(99999999999999999999999999999999.99, 4)")
-        checkSparkAnswerAndNumOfAggregates(s"select sum(a) from $table group by b order by b", 2)
+        checkSparkAnswerAndNumOfAggregates(
+          s"select avg(a), sum(a) from $table group by b order by b",
+          2)
       }
     }
   }
@@ -998,7 +1019,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   test("final decimal avg") {
     withSQLConf(
       CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
-      CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true",
+      CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true",
       CometConf.COMET_SHUFFLE_MODE.key -> "native") {
       Seq(true, false).foreach { dictionaryEnabled =>
         withSQLConf("parquet.enable.dictionary" -> dictionaryEnabled.toString) {
@@ -1069,7 +1090,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         withSQLConf(
           CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> nativeShuffleEnabled.toString,
           CometConf.COMET_SHUFFLE_MODE.key -> "native",
-          CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true") {
+          CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
           withTempDir { dir =>
             val path = new Path(dir.toURI.toString, "test")
             makeParquetFile(path, 1000, 20, dictionaryEnabled)
