@@ -86,67 +86,82 @@ abstract class ParquetReadSuite extends CometTestBase {
   }
 
   test("unsupported Spark types") {
-    // for native iceberg compat, CometScanExec supports some types that native_comet does not.
-    // note that native_datafusion does not use CometScanExec so we need not include that in
-    // the check
-    val isDataFusionScan = usingDataSourceExec(conf)
-    Seq(
-      NullType -> false,
-      BooleanType -> true,
-      ByteType -> true,
-      ShortType -> true,
-      IntegerType -> true,
-      LongType -> true,
-      FloatType -> true,
-      DoubleType -> true,
-      BinaryType -> true,
-      StringType -> true,
-      // Timestamp here arbitrary for picking a concrete data type to from ArrayType
-      // Any other type works
-      ArrayType(TimestampType) -> isDataFusionScan,
-      StructType(
-        Seq(
-          StructField("f1", DecimalType.SYSTEM_DEFAULT),
-          StructField("f2", StringType))) -> isDataFusionScan,
-      MapType(keyType = LongType, valueType = DateType) -> isDataFusionScan,
-      StructType(
-        Seq(StructField("f1", ByteType), StructField("f2", StringType))) -> isDataFusionScan,
-      MapType(keyType = IntegerType, valueType = BinaryType) -> isDataFusionScan)
-      .foreach { case (dt, expected) =>
-        val fallbackReasons = new ListBuffer[String]()
-        assert(
-          CometScanTypeChecker(CometConf.COMET_NATIVE_SCAN_IMPL.get())
-            .isTypeSupported(dt, "", fallbackReasons) == expected)
-        // usingDataFusionParquetExec does not support CometBatchScanExec yet
-        if (!isDataFusionScan) {
-          assert(CometBatchScanExec.isTypeSupported(dt, "", fallbackReasons) == expected)
+    // TODO this test is not correctly implemented for scan implementations other than SCAN_NATIVE_COMET
+    // https://github.com/apache/datafusion-comet/issues/2188
+    withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> CometConf.SCAN_NATIVE_COMET) {
+      // for native iceberg compat, CometScanExec supports some types that native_comet does not.
+      // note that native_datafusion does not use CometScanExec so we need not include that in
+      // the check
+      val isDataFusionScan = usingDataSourceExec(conf)
+      Seq(
+        NullType -> false,
+        BooleanType -> true,
+        ByteType -> true,
+        ShortType -> true,
+        IntegerType -> true,
+        LongType -> true,
+        FloatType -> true,
+        DoubleType -> true,
+        BinaryType -> true,
+        StringType -> true,
+        // Timestamp here arbitrary for picking a concrete data type to from ArrayType
+        // Any other type works
+        ArrayType(TimestampType) -> isDataFusionScan,
+        StructType(
+          Seq(
+            StructField("f1", DecimalType.SYSTEM_DEFAULT),
+            StructField("f2", StringType))) -> isDataFusionScan,
+        MapType(keyType = LongType, valueType = DateType) -> isDataFusionScan,
+        StructType(
+          Seq(StructField("f1", ByteType), StructField("f2", StringType))) -> isDataFusionScan,
+        MapType(keyType = IntegerType, valueType = BinaryType) -> isDataFusionScan)
+        .foreach { case (dt, expected) =>
+          val fallbackReasons = new ListBuffer[String]()
+          // TODO CometScanTypeChecker should only be used for ParquetReadSuiteV1Suite
+          assert(
+            CometScanTypeChecker(CometConf.COMET_NATIVE_SCAN_IMPL.get())
+              .isTypeSupported(dt, "", fallbackReasons) == expected)
+          // usingDataFusionParquetExec does not support CometBatchScanExec yet
+          // TODO CometBatchScanExec should only be used for ParquetReadSuiteV2Suite
+          if (!isDataFusionScan) {
+            assert(CometBatchScanExec.isTypeSupported(dt, "", fallbackReasons) == expected)
+          }
         }
-      }
+    }
   }
 
   test("unsupported Spark schema") {
-    val schemaDDLs =
-      Seq("f1 int, f2 boolean", "f1 int, f2 array<int>", "f1 map<long, string>, f2 array<double>")
-        .map(s => StructType.fromDDL(s))
+    // TODO this test is not correctly implemented for scan implementations other than SCAN_NATIVE_COMET
+    // https://github.com/apache/datafusion-comet/issues/2188
+    withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> CometConf.SCAN_NATIVE_COMET) {
+      val schemaDDLs =
+        Seq(
+          "f1 int, f2 boolean",
+          "f1 int, f2 array<int>",
+          "f1 map<long, string>, f2 array<double>")
+          .map(s => StructType.fromDDL(s))
 
-    // Arrays support for iceberg compat native and for Parquet V1
-    val cometScanExecSupported =
-      if (usingDataSourceExec(conf) && this.isInstanceOf[ParquetReadV1Suite])
-        Seq(true, true, true)
-      else Seq(true, false, false)
+      // Arrays support for iceberg compat native and for Parquet V1
+      val cometScanExecSupported =
+        if (usingDataSourceExec(conf) && this.isInstanceOf[ParquetReadV1Suite])
+          Seq(true, true, true)
+        else Seq(true, false, false)
 
-    val cometBatchScanExecSupported = Seq(true, false, false)
-    val fallbackReasons = new ListBuffer[String]()
+      val cometBatchScanExecSupported = Seq(true, false, false)
+      val fallbackReasons = new ListBuffer[String]()
 
-    schemaDDLs.zip(cometScanExecSupported).foreach { case (schema, expected) =>
-      assert(
-        CometScanTypeChecker(CometConf.COMET_NATIVE_SCAN_IMPL.get())
-          .isSchemaSupported(StructType(schema), fallbackReasons) == expected)
-    }
+      // TODO CometScanTypeChecker should only be used for ParquetReadSuiteV1Suite
+      schemaDDLs.zip(cometScanExecSupported).foreach { case (schema, expected) =>
+        assert(
+          CometScanTypeChecker(CometConf.COMET_NATIVE_SCAN_IMPL.get())
+            .isSchemaSupported(StructType(schema), fallbackReasons) == expected)
+      }
 
-    schemaDDLs.zip(cometBatchScanExecSupported).foreach { case (schema, expected) =>
-      assert(
-        CometBatchScanExec.isSchemaSupported(StructType(schema), fallbackReasons) == expected)
+      // TODO CometBatchScanExec should only be used for ParquetReadSuiteV2Suite
+      schemaDDLs.zip(cometBatchScanExecSupported).foreach { case (schema, expected) =>
+        assert(
+          CometBatchScanExec.isSchemaSupported(StructType(schema), fallbackReasons) == expected)
+      }
     }
   }
 
@@ -354,8 +369,6 @@ abstract class ParquetReadSuite extends CometTestBase {
   }
 
   test("test multiple pages with different sizes and nulls") {
-    // https://github.com/apache/datafusion-comet/issues/1441
-    assume(!usingDataSourceExec)
     def makeRawParquetFile(
         path: Path,
         dictionaryEnabled: Boolean,
@@ -425,40 +438,42 @@ abstract class ParquetReadSuite extends CometTestBase {
       expected
     }
 
-    Seq(64, 128, 256, 512, 1024, 4096, 5000).foreach { pageSize =>
-      withTempDir { dir =>
-        val path = new Path(dir.toURI.toString, "part-r-0.parquet")
-        val expected = makeRawParquetFile(path, dictionaryEnabled = false, 10000, pageSize)
-        readParquetFile(path.toString) { df =>
-          checkAnswer(
-            df,
-            expected.map {
-              case None =>
-                Row(null, null, null, null, null, null, null, null, null, null, null, null, null,
-                  null)
-              case Some(i) =>
-                val flba_field = Array.fill(3)(i % 10 + 48) // char '0' is 48 in ascii
-                Row(
-                  i % 2 == 0,
-                  i.toByte,
-                  i.toShort,
-                  i,
-                  i.toLong,
-                  i.toFloat,
-                  i.toDouble,
-                  i.toString * 48,
-                  (-i).toByte,
-                  (-i).toShort,
-                  java.lang.Integer.toUnsignedLong(-i),
-                  new BigDecimal(UnsignedLong.fromLongBits((-i).toLong).bigIntegerValue()),
-                  i.toString,
-                  flba_field)
-            })
-        }
-        readParquetFile(path.toString) { df =>
-          assert(
-            df.filter("_8 IS NOT NULL AND _4 % 256 == 255").count() ==
-              expected.flatten.count(_ % 256 == 255))
+    withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> CometConf.SCAN_NATIVE_COMET) {
+      Seq(64, 128, 256, 512, 1024, 4096, 5000).foreach { pageSize =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "part-r-0.parquet")
+          val expected = makeRawParquetFile(path, dictionaryEnabled = false, 10000, pageSize)
+          readParquetFile(path.toString) { df =>
+            checkAnswer(
+              df,
+              expected.map {
+                case None =>
+                  Row(null, null, null, null, null, null, null, null, null, null, null, null,
+                    null, null)
+                case Some(i) =>
+                  val flba_field = Array.fill(3)(i % 10 + 48) // char '0' is 48 in ascii
+                  Row(
+                    i % 2 == 0,
+                    i.toByte,
+                    i.toShort,
+                    i,
+                    i.toLong,
+                    i.toFloat,
+                    i.toDouble,
+                    i.toString * 48,
+                    (-i).toByte,
+                    (-i).toShort,
+                    java.lang.Integer.toUnsignedLong(-i),
+                    new BigDecimal(UnsignedLong.fromLongBits((-i).toLong).bigIntegerValue()),
+                    i.toString,
+                    flba_field)
+              })
+          }
+          readParquetFile(path.toString) { df =>
+            assert(
+              df.filter("_8 IS NOT NULL AND _4 % 256 == 255").count() ==
+                expected.flatten.count(_ % 256 == 255))
+          }
         }
       }
     }
@@ -1330,8 +1345,6 @@ abstract class ParquetReadSuite extends CometTestBase {
   }
 
   test("scan metrics") {
-    // https://github.com/apache/datafusion-comet/issues/1441
-    assume(CometConf.COMET_NATIVE_SCAN_IMPL.get() != CometConf.SCAN_NATIVE_ICEBERG_COMPAT)
 
     val cometScanMetricNames = Seq(
       "ParquetRowGroups",
@@ -1351,30 +1364,34 @@ abstract class ParquetReadSuite extends CometTestBase {
       "time_elapsed_scanning_until_data")
 
     withParquetTable((0 until 10000).map(i => (i, i.toDouble)), "tbl") {
-      val df = sql("SELECT * FROM tbl WHERE _1 > 0")
-      val scans = df.queryExecution.executedPlan collect {
-        case s: CometScanExec => s
-        case s: CometBatchScanExec => s
-        case s: CometNativeScanExec => s
-      }
-      assert(scans.size == 1, s"Expect one scan node but found ${scans.size}")
-      val metrics = scans.head.metrics
+      // TODO need to implement metrics for SCAN_NATIVE_ICEBERG_COMPAT
+      // https://github.com/apache/datafusion-comet/issues/1882
+      withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> CometConf.SCAN_NATIVE_COMET) {
+        val df = sql("SELECT * FROM tbl WHERE _1 > 0")
+        val scans = df.queryExecution.executedPlan collect {
+          case s: CometScanExec => s
+          case s: CometBatchScanExec => s
+          case s: CometNativeScanExec => s
+        }
+        assert(scans.size == 1, s"Expect one scan node but found ${scans.size}")
+        val metrics = scans.head.metrics
 
-      val metricNames = scans.head match {
-        case _: CometNativeScanExec => cometNativeScanMetricNames
-        case _ => cometScanMetricNames
-      }
+        val metricNames = scans.head match {
+          case _: CometNativeScanExec => cometNativeScanMetricNames
+          case _ => cometScanMetricNames
+        }
 
-      metricNames.foreach { metricName =>
-        assert(metrics.contains(metricName), s"metric $metricName was not found")
-      }
+        metricNames.foreach { metricName =>
+          assert(metrics.contains(metricName), s"metric $metricName was not found")
+        }
 
-      df.collect()
+        df.collect()
 
-      metricNames.foreach { metricName =>
-        assert(
-          metrics(metricName).value > 0,
-          s"Expect metric value for $metricName to be positive")
+        metricNames.foreach { metricName =>
+          assert(
+            metrics(metricName).value > 0,
+            s"Expect metric value for $metricName to be positive")
+        }
       }
     }
   }
@@ -1848,16 +1865,16 @@ class ParquetReadV1Suite extends ParquetReadSuite with AdaptiveSparkPlanHelper {
   test("Test V1 parquet scan uses respective scanner") {
     Seq(
       ("false", CometConf.SCAN_NATIVE_COMET, "FileScan parquet"),
-      ("true", CometConf.SCAN_NATIVE_COMET, "CometScan parquet"),
+      ("true", CometConf.SCAN_NATIVE_COMET, "CometScan [native_comet] parquet"),
       ("true", CometConf.SCAN_NATIVE_DATAFUSION, "CometNativeScan"),
-      ("true", CometConf.SCAN_NATIVE_ICEBERG_COMPAT, "CometScan parquet")).foreach {
-      case (cometEnabled, cometNativeScanImpl, expectedScanner) =>
+      ("true", CometConf.SCAN_NATIVE_ICEBERG_COMPAT, "CometScan [native_iceberg_compat] parquet"))
+      .foreach { case (cometEnabled, cometNativeScanImpl, expectedScanner) =>
         testScanner(
           cometEnabled,
           cometNativeScanImpl,
           scanner = expectedScanner,
           v1 = Some("parquet"))
-    }
+      }
   }
 
   test("test V1 parquet native scan -- case insensitive") {
@@ -1970,6 +1987,7 @@ class ParquetReadV1Suite extends ParquetReadSuite with AdaptiveSparkPlanHelper {
                       "optional_map.key, " +
                       "optional_map.value, " +
                       "map_keys(complex_map), " +
+                      "map_entries(complex_map), " +
                       "map_values(complex_map) " +
                       "from complex_types"))
                 // leaf fields
@@ -1982,6 +2000,8 @@ class ParquetReadV1Suite extends ParquetReadSuite with AdaptiveSparkPlanHelper {
                       "optional_map.value, " +
                       "map_keys(complex_map)[0].key_field1, " +
                       "map_keys(complex_map)[0].key_field2, " +
+                      "map_entries(complex_map)[0].key, " +
+                      "map_entries(complex_map)[0].value, " +
                       "map_values(complex_map)[0].value_field1, " +
                       "map_values(complex_map)[0].value_field2 " +
                       "from complex_types"))
