@@ -29,6 +29,8 @@ import org.apache.spark.sql.catalyst.util.{fileToString, resourceToString, strin
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.test.TestSparkSession
 
+import org.apache.comet.CometConf
+
 /**
  * Because we need to modify some methods of Spark `TPCDSQueryTestSuite` but they are private, we
  * copy Spark `TPCDSQueryTestSuite`.
@@ -164,12 +166,18 @@ class CometTPCDSQueryTestSuite extends QueryTest with TPCDSBase with CometSQLQue
     }
   }
 
+  val baseConf: Map[String, String] = Map(
+    CometConf.COMET_ENABLED.key -> "true",
+    CometConf.COMET_NATIVE_SCAN_ENABLED.key -> "true",
+    CometConf.COMET_EXEC_ENABLED.key -> "true",
+    CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true")
+
   val sortMergeJoinConf: Map[String, String] = Map(
     SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "-1",
     SQLConf.PREFER_SORTMERGEJOIN.key -> "true")
 
-  val broadcastHashJoinConf: Map[String, String] = Map(
-    SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "10485760")
+  val broadcastHashJoinConf: Map[String, String] =
+    Map(SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "10485760")
 
   val shuffledHashJoinConf: Map[String, String] = Map(
     SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key -> "-1",
@@ -213,7 +221,7 @@ class CometTPCDSQueryTestSuite extends QueryTest with TPCDSBase with CometSQLQue
           // that can cause OOM in GitHub Actions
           if (!(sortMergeJoin && name == "q72")) {
             System.gc() // Workaround for GitHub Actions memory limitation, see also SPARK-37368
-            runQuery(queryString, goldenFile, conf)
+            runQuery(queryString, goldenFile, baseConf ++ conf)
           }
         }
       }
@@ -231,7 +239,7 @@ class CometTPCDSQueryTestSuite extends QueryTest with TPCDSBase with CometSQLQue
           // that can cause OOM in GitHub Actions
           if (!(sortMergeJoin && name == "q72")) {
             System.gc() // SPARK-37368
-            runQuery(queryString, goldenFile, conf)
+            runQuery(queryString, goldenFile, baseConf ++ conf)
           }
         }
       }
@@ -245,7 +253,7 @@ class CometTPCDSQueryTestSuite extends QueryTest with TPCDSBase with CometSQLQue
         val goldenFile = new File(s"$baseResourcePath/extended", s"$name.sql.out")
         joinConfs.foreach { conf =>
           System.gc() // SPARK-37368
-          runQuery(queryString, goldenFile, conf)
+          runQuery(queryString, goldenFile, baseConf ++ conf)
         }
       }
     }
